@@ -17,6 +17,8 @@
 #include <errno.h>
 #include <stdio.h>
 
+#define IIO_MIN_SAMPLING_FREQUENCY	10 /* Hz */
+
 /**
  * iio_channel_info - information about a given channel
  * @name: channel name
@@ -195,6 +197,8 @@ iioutils_get_param_float (float      *output,
 	char *builtname, *filename;
 	int ret = 0;
 
+	g_debug ("Trying to read '%s_%s' (name) from dir '%s'", name, param_name, device_dir);
+
 	builtname = g_strdup_printf ("%s_%s", name, param_name);
 	filename = g_build_filename (device_dir, builtname, NULL);
 	g_free (builtname);
@@ -207,8 +211,11 @@ iioutils_get_param_float (float      *output,
 		return 0;
 	}
 
+	g_debug ("Failed to read float from %s: %s", filename, g_strerror (-ret));
 	g_free (filename);
 
+	g_debug ("Trying to read '%s_%s' (generic name) from dir '%s'", generic_name, param_name, device_dir);
+
 	builtname = g_strdup_printf ("%s_%s", generic_name, param_name);
 	filename = g_build_filename (device_dir, builtname, NULL);
 	g_free (builtname);
@@ -221,6 +228,8 @@ iioutils_get_param_float (float      *output,
 		ret = -errno;
 		if (ret != -ENOENT)
 			g_warning ("Failed to read float from %s: %s", filename, g_strerror (-ret));
+		else
+			g_debug ("Failed to read float from %s: %s", filename, g_strerror (-ret));
 	}
 
 	g_free (filename);
@@ -517,13 +526,16 @@ size_from_channelarray (iio_channel_info **channels,
  **/
 void
 process_scan_1 (char              *data,
-		BufferDrvData        *buffer_data,
+		BufferDrvData     *buffer_data,
 		const char        *ch_name,
 		int               *ch_val,
 		gdouble           *ch_scale,
 		gboolean          *ch_present)
 {
 	int k;
+
+	*ch_scale = 1.0;
+
 	for (k = 0; k < buffer_data->channels_count; k++) {
 		if (strcmp (buffer_data->channels[k]->name, ch_name) != 0)
 			continue;
@@ -540,6 +552,8 @@ process_scan_1 (char              *data,
 				if (buffer_data->channels[k]->bits_used < 32)
 					val &= ((guint32) 1 << buffer_data->channels[k]->bits_used) - 1;
 				*ch_val = (int) val;
+				if (buffer_data->channels[k]->scale)
+					*ch_scale = buffer_data->channels[k]->scale;
 				*ch_present = TRUE;
 			} else {
 				gint32 val = iio_read32(buffer_data->channels[k], (gint8 *) data);
@@ -550,8 +564,6 @@ process_scan_1 (char              *data,
 				*ch_val = (int) val;
 				if (buffer_data->channels[k]->scale)
 					*ch_scale = buffer_data->channels[k]->scale;
-				else
-					*ch_scale = 1.0;
 				*ch_present = TRUE;
 			}
 			break;
@@ -568,6 +580,51 @@ process_scan_1 (char              *data,
 		g_warning ("IIO channel '%s' could not be found", ch_name);
 }
 
+/**
+ * iio_fixup_sampling_frequency: Fixup devices *sampling_frequency attributes
+ * @dev: the IIO device to fix the sampling frequencies for
+ *
+ * Make sure devices with *sampling_frequency attributes are sampling at
+ * 10Hz or more. This fixes 2 problems:
+ * 1) Some buffered devices default their sampling_frequency to 0Hz and then
+ * never produce any readings.
+ * 2) Some polled devices default to 1Hz and wait for a fresh sample before
+ * returning from sysfs *_raw reads, blocking all of iio-sensor-proxy for
+ * multiple seconds
+ **/
+gboolean
+iio_fixup_sampling_frequency (GUdevDevice *dev)
+{
+	GDir *dir;
+	const char *device_dir;
+	const char *name;
+	GError *error = NULL;
+	double sample_freq;
+
+	device_dir = g_udev_device_get_sysfs_path (dev);
+	dir = g_dir_open (g_udev_device_get_sysfs_path (dev), 0, &error);
+	if (!dir) {
+		g_warning ("Failed to open directory '%s': %s", device_dir, error->message);
+		g_error_free (error);
+		return FALSE;
+	}
+
+	while ((name = g_dir_read_name (dir))) {
+		if (g_str_has_suffix (name, "sampling_frequency") == FALSE)
+			continue;
+
+		sample_freq = g_udev_device_get_sysfs_attr_as_double (dev, name);
+		if (sample_freq >= IIO_MIN_SAMPLING_FREQUENCY)
+			continue; /* Continue with pre-set sample freq. */
+
+		/* Sample freq too low, set it to 10Hz */
+		if (write_sysfs_int (name, device_dir, IIO_MIN_SAMPLING_FREQUENCY) < 0)
+			g_warning ("Could not fix sample-freq for %s/%s", device_dir, name);
+	}
+	g_dir_close (dir);
+	return TRUE;
+}
+
 /**
  * enable_sensors: enable all the sensors in a device
  * @device_dir: the IIO device directory in sysfs
@@ -719,7 +776,8 @@ buffer_drv_data_new (GUdevDevice *device,
 	buffer_data->trigger_name = g_strdup (trigger_name);
 	buffer_data->device = g_object_ref (device);
 
-	if (!enable_sensors (device, 1) ||
+	if (!iio_fixup_sampling_frequency (device) ||
+	    !enable_sensors (device, 1) ||
 	    !enable_trigger (buffer_data) ||
 	    !enable_ring_buffer (buffer_data) ||
 	    !build_channels (buffer_data)) {
